@@ -23,8 +23,6 @@ def orm(dba):
     from tina4_python import root_path
     Debug("Initializing ORM")
     orm_path = root_path + os.sep + "src" + os.sep + "orm"
-    print("Path", orm_path)
-
     if not os.path.exists(orm_path):
         os.makedirs(orm_path)
 
@@ -260,6 +258,7 @@ class ORM:
         return snake_case_name
 
     def __init__(self, init_object=None, table_name=None):
+        from tina4_python import root_path
         # save the initial declarations
         counter = 0
         self.__field_definitions__ = {}
@@ -281,26 +280,34 @@ class ORM:
         if init_object is not None:
             self.__populate_orm(init_object)
 
-        Debug("Checking for", self.__table_name__, TINA4_LOG_INFO)
+        # Debug("Checking for", self.__table_name__, TINA4_LOG_INFO)
         if self.__dba__:
             self.__table_exists = self.__dba__.table_exists(self.__table_name__)
             if not self.__table_exists:
                 sql = self.__create_table__(self.__table_name__)
-                filename = tina4_python.root_path + os.sep + "migrations" + os.sep + "__" + self.__table_name__ + ".sql"
+                filename = root_path + os.sep + "migrations" + os.sep + "__" + self.__table_name__ + ".sql"
                 os.makedirs(os.path.dirname(filename), exist_ok=True)
                 with open(filename, "w") as f:
                     f.write(sql)
                     f.close()
-            Debug("Table Exists", self.__table_exists, TINA4_LOG_INFO)
+            # Debug("Table Exists", self.__table_exists, TINA4_LOG_INFO)
         else:
             self.__table_exists = False
 
+        self.__populate_orm({})
+
     def __populate_orm(self, init_object):
         """
         Populates an ORM object from an input object, also transforms camel case objects to snake case ...
         :param init_object:
         :return:
         """
+        for field, field_definition in self.__field_definitions__.items():
+            if hasattr(self, field):
+                setattr(self, field, None)
+                field_definition.value = None
+                setattr(self, field, field_definition)
+
         if isinstance(init_object, str):
             init_object = json.loads(init_object)
 
@@ -339,11 +346,17 @@ class ORM:
                     data[key] = current_value.value
                 else:
                     if not isinstance(current_value, ForeignKeyField) and value.auto_increment:
-                        data[key] = self.__dba__.get_next_id(table_name=self.__table_name__, column_name=value.column_name)
+                        new_id = self.__dba__.get_next_id(table_name=self.__table_name__, column_name=value.column_name)
+                        if new_id is not None:
+                            data[key] = new_id
+                            current_value.value = data[key]
+                        print("GET INCREMENT", data)
                     else:
                         data[key] = current_value.default_value
             else:
                 data[key] = current_value
+
+
         return data
 
     def __str__(self):
@@ -419,6 +432,7 @@ class ORM:
         if not self.__table_exists:
             Debug("ORM: Load Error - Table", self.__table_name__, "does not exist", TINA4_LOG_ERROR)
             return False
+
         if query == "":
             sql = f"select * from {self.__table_name__} where "
             primary_keys = self.__get_primary_keys()
@@ -433,6 +447,7 @@ class ORM:
             sql = f"select * from {self.__table_name__} where {query}"
 
         if self.__dba__ is not None:
+            print("FETCHING", sql)
             record = self.__dba__.fetch_one(sql, params)
             try:
                 if record:
@@ -440,6 +455,7 @@ class ORM:
                         if key in self.__field_definitions__:
                             field_value = self.__field_definitions__[key]
                             field_value.value = value
+
                             setattr(self, key, field_value)
                     return True
                 else:
@@ -460,6 +476,8 @@ class ORM:
             return False
         # check if record exists
         data = self.to_dict()
+
+        print ("DATA BEFORE", data)
         primary_keys = self.__get_primary_keys()
         sql = "select count(*) as \"count_records\" from " + self.__table_name__ + " where "
         counter = 0
@@ -476,14 +494,18 @@ class ORM:
             record = self.__dba__.fetch_one(sql, input_params)
 
             if record["count_records"] == 0:
+                print("NEW")
                 result = self.__dba__.insert(self.__table_name__, data)
             else:
+                print("UPDATE")
                 result = self.__dba__.update(self.__table_name__, data)
 
             self.__dba__.commit()
 
             if result:
                 self.load()
+
+                print ("DATA AFTER", self.to_dict())
                 return True
         except Exception as e:
             Debug.error("Error saving", str(e))
@@ -508,10 +530,11 @@ class ORM:
         else:
             sql = f"delete from {self.__table_name__} where {query}"
 
+        result = False
         if self.__dba__ is not None:
-            result = self.__dba__.delete(sql, params)
+            result = self.__dba__.execute(sql, params)
 
-        if not result:
+        if result.error is not None:
             self.__dba__.rollback()
             return False
         else:
