@@ -9,16 +9,21 @@ import base64
 import json
 import os
 import re
+import mimetypes
 from urllib.parse import unquote_plus
 from urllib.parse import urlparse, parse_qsl
 import tina4_python
 from tina4_python import Constant
-from tina4_python.Constant import HTTP_REDIRECT
+from tina4_python.Constant import HTTP_REDIRECT, HTTP_OK, HTTP_SERVER_ERROR
 from tina4_python.Session import Session
+from tina4_python.Router import Router
+from tina4_python.Debug import Debug
+from tina4_python.Template import Template
+
 
 def is_int(v):
     try:
-        f=int(v)
+        f = int(v)
     except ValueError:
         return False
     return True
@@ -27,7 +32,7 @@ def is_int(v):
 class Webserver:
     async def get_content_length(self):
         # get the content length
-        if "content-length" in self.lowercase_headers != -1:
+        if "content-length" in self.lowercase_headers:
             return int(self.lowercase_headers["content-length"])
 
         return 0
@@ -93,7 +98,16 @@ class Webserver:
 
         return {"data": base64.encodebytes(content).decode("utf-8").replace("\n", "")}
 
-    async def get_response(self, method):
+    async def send_basic_headers(self, headers):
+        self.send_header("Access-Control-Allow-Origin", "*", headers)
+        self.send_header("Access-Control-Allow-Headers",
+                         "Origin, X-Requested-With, Content-Type, Accept, Authorization", headers)
+        self.send_header("Access-Control-Allow-Credentials", "True", headers)
+        # self.send_header("Content-Length", str(len(response.content)), headers)
+        self.send_header("Connection", "Keep-Alive", headers)
+        self.send_header("Keep-Alive", "timeout=5, max=30", headers)
+
+    async def get_response(self, method, transport, asgi_response=False):
         """
         Get response
         :param method: GET, POST, PATCH, DELETE, PUT
@@ -107,7 +121,11 @@ class Webserver:
             self.send_header("Access-Control-Allow-Credentials", "True", headers)
 
             headers = await self.get_headers(headers, self.response_protocol, Constant.HTTP_OK)
-            return headers
+
+            if asgi_response:
+                return None, headers
+            else:
+                return headers
 
         params = dict(parse_qsl(urlparse(self.path).query, keep_blank_values=True))
 
@@ -134,13 +152,13 @@ class Webserver:
                         if isinstance(start_var, dict) and var_name in start_var:
                             start_var = start_var[var_name]
                         else:
-                            if counter+1 < len(var_names) and is_int(var_names[counter+1]) :
+                            if counter + 1 < len(var_names) and is_int(var_names[counter + 1]):
                                 if var_name not in start_var:
                                     start_var[var_name] = []
                                 start_var = start_var[var_name]
                             else:
-                                if counter-1 > 0 and is_int(var_names[counter-1]):
-                                    index = int(var_names[counter-1])
+                                if counter - 1 > 0 and is_int(var_names[counter - 1]):
+                                    index = int(var_names[counter - 1])
                                     new_value = {var_name: value}
                                     if index in range(len(start_var)):
                                         start_var[index].update(new_value)
@@ -151,7 +169,7 @@ class Webserver:
                                     start_var = start_var[index]
                                 else:
                                     if isinstance(start_var, dict):
-                                        if counter+1 == len(var_names):
+                                        if counter + 1 == len(var_names):
                                             start_var[var_name] = value
                                         else:
                                             start_var[var_name] = {}
@@ -167,22 +185,17 @@ class Webserver:
         else:
             body = None
 
-        request = {"params": params, "body": body, "raw_data": self.request, "url": self.path,
-                   "headers": self.lowercase_headers, "raw_request": self.request_raw, "raw_content": self.content_raw}
+        request = {"params": params, "body": body, "raw_data": self.request, "url": self.path, "session": self.session,
+                   "headers": self.lowercase_headers, "raw_request": self.request_raw, "raw_content": self.content_raw,
+                   "transport": transport, "asgi_response": asgi_response}
 
         tina4_python.tina4_current_request = request
 
         response = await self.router_handler.resolve(method, self.path, request, self.lowercase_headers, self.session)
 
         if HTTP_REDIRECT != response.http_code:
-            self.send_header("Access-Control-Allow-Origin", "*", headers)
-            self.send_header("Access-Control-Allow-Headers",
-                             "Origin, X-Requested-With, Content-Type, Accept, Authorization", headers)
-            self.send_header("Access-Control-Allow-Credentials", "True", headers)
             self.send_header("Content-Type", response.content_type, headers)
-            # self.send_header("Content-Length", str(len(response.content)), headers)
-            self.send_header("Connection", "Keep-Alive", headers)
-            self.send_header("Keep-Alive", "timeout=5, max=30", headers)
+            await self.send_basic_headers(headers)
 
             if os.getenv("TINA4_SESSION", "PY_SESS") in self.cookies:
                 self.send_header("Set-Cookie",
@@ -193,6 +206,9 @@ class Webserver:
         for response_header in response.headers:
             self.send_header(response_header, response.headers[response_header], headers)
 
+        if asgi_response:
+            return response, headers
+
         headers = await self.get_headers(headers, self.response_protocol, response.http_code)
 
         if isinstance(response.content, str):
@@ -215,9 +231,8 @@ class Webserver:
         return headers.encode()
 
     async def run_server(self):
-        server = await asyncio.start_server(self.handle_client, self.host_name, self.port)
-        async with server:
-            await server.serve_forever()
+        self.server = await asyncio.start_server(self.handle_client, self.host_name, self.port)
+        await self.server.serve_forever()
 
     async def get_data(self, reader):
         try:
@@ -258,55 +273,114 @@ class Webserver:
         return protocol, headers, lowercase_headers, content, raw_data, content_data
 
     async def handle_client(self, reader, writer):
-        # Get the client request
-        protocol, headers_list, lowercase_headers, request, request_raw, content_raw = await self.get_data(reader)
-        # Strange blank request ?
-        if protocol == '':
-            return
-        # Decode the request
-        self.request_raw = request_raw
-        self.content_raw = content_raw
-        self.request = request
-        self.headers = headers_list
-        self.lowercase_headers = lowercase_headers
+        try:
+            # Get the client request
+            protocol, headers_list, lowercase_headers, request, request_raw, content_raw = await self.get_data(reader)
+            # Strange blank request ?
+            if protocol == '':
+                return
+            # Decode the request
+            self.request_raw = request_raw
+            self.content_raw = content_raw
+            self.request = request
+            self.headers = headers_list
+            self.lowercase_headers = lowercase_headers
 
-        protocol = protocol.split(" ")
-        # print(protocol, headers_list)
-        self.method = protocol[0]
-        self.path = protocol[1]
+            protocol = protocol.split(" ")
+            # print(protocol, headers_list)
+            self.method = protocol[0]
+            self.path = protocol[1]
 
-        # parse cookies
-        cookie_list = {}
-        if "cookie" in self.lowercase_headers:
-            cookie_list_temp = self.lowercase_headers["cookie"].split(";")
-            for cookie_value in cookie_list_temp:
-                cookie = cookie_value.split("=", 1)
-                cookie_list[cookie[0].strip()] = cookie[1].strip()
 
-        self.cookies = cookie_list
+            method_list = [Constant.TINA4_GET, Constant.TINA4_DELETE, Constant.TINA4_PUT, Constant.TINA4_ANY,
+                           Constant.TINA4_POST, Constant.TINA4_PATCH, Constant.TINA4_OPTIONS]
 
-        # initialize the session
-        self.session = Session(os.getenv("TINA4_SESSION", "PY_SESS"),
-                               os.getenv("TINA4_SESSION_FOLDER", tina4_python.root_path + os.sep + "sessions"),
-                               os.getenv("TINA4_SESSION_HANDLER", "SessionFileHandler")
-                               )
+            contains_method = [ele for ele in method_list if (ele in self.method)]
 
-        if os.getenv("TINA4_SESSION", "PY_SESS") in self.cookies:
-            self.session.load(self.cookies[os.getenv("TINA4_SESSION", "PY_SESS")])
-        else:
-            self.cookies[os.getenv("TINA4_SESSION", "PY_SESS")] = self.session.start()
+            request_handled = False
+            # return static content asap
+            if self.method == "GET" and "sec-websocket-key" not in self.lowercase_headers:
+                # split URL and extract query string
+                url = Router.clean_url(self.path)
+                url_parts = url.split('?')
+                url = url_parts[0]
 
-        method_list = [Constant.TINA4_GET, Constant.TINA4_DELETE, Constant.TINA4_PUT, Constant.TINA4_ANY,
-                       Constant.TINA4_POST, Constant.TINA4_PATCH, Constant.TINA4_OPTIONS]
+                # Serve statics
+                static_file = tina4_python.root_path + os.sep + "src" + os.sep + "public" + url.replace("/", os.sep)
+                Debug.info("Attempting to serve static file: " + static_file)
+                if os.path.isfile(static_file):
+                    mime_type = mimetypes.guess_type(url)[0]
+                    with open(static_file, 'rb') as file:
+                        headers = []
+                        self.send_header("Content-Type", mime_type, headers)
+                        await self.send_basic_headers(headers)
+                        content = file.read()
+                        headers = await self.get_headers(headers, self.response_protocol, HTTP_OK)
+                        writer.write(headers + content)
+                        await writer.drain()
+                        await writer.close()
+                        request_handled = True
 
-        contains_method = [ele for ele in method_list if (ele in self.method)]
+            if not request_handled:
+                # parse cookies
+                cookie_list = {}
+                content = ""
+                if "cookie" in self.lowercase_headers:
+                    cookie_list_temp = self.lowercase_headers["cookie"].split(";")
+                    for cookie_value in cookie_list_temp:
+                        cookie = cookie_value.split("=", 1)
+                        cookie_list[cookie[0].strip()] = cookie[1].strip()
 
-        if self.method != "" and contains_method:
-            content = await (self.get_response(self.method))
-            writer.write(content)
+                self.cookies = cookie_list
+
+                # initialize the session
+                self.session = Session(os.getenv("TINA4_SESSION", "PY_SESS"),
+                                       os.getenv("TINA4_SESSION_FOLDER", tina4_python.root_path + os.sep + "sessions"),
+                                       os.getenv("TINA4_SESSION_HANDLER", "SessionFileHandler")
+                                       )
+
+                if os.getenv("TINA4_SESSION", "PY_SESS") in self.cookies:
+                    self.session.load(self.cookies[os.getenv("TINA4_SESSION", "PY_SESS")])
+                else:
+                    self.cookies[os.getenv("TINA4_SESSION", "PY_SESS")] = self.session.start()
+
+                if "sec-websocket-key" not in self.lowercase_headers:
+                    try:
+                        if self.method != "" and contains_method:
+                            content = await (self.get_response(self.method, writer))
+                        if content != "":
+                            writer.write(content)
+                            await writer.drain()
+                        writer.close()
+                    except BrokenPipeError as e:
+                        # socket got terminated
+                        pass
+                else:
+                    # for sockets
+                    await (self.get_response(self.method, writer))
+
+
+        except Exception as e:
+            error_string = tina4_python.global_exception_handler(e)
+            headers = []
+            await self.send_basic_headers(headers)
+            headers = await self.get_headers(headers, self.response_protocol, HTTP_SERVER_ERROR)
+            url = Router.clean_url(self.path)
+
+            content_type = "text/html"
+            if "content-type" in self.lowercase_headers:
+                content_type = self.lowercase_headers["content-type"].lower()
+
+            if content_type == "application/json":
+                html = json.dumps({"error": "500 - Internal Server Error",
+                                   "data": {"server": {"url": url}, "error_message": error_string}})
+            else:
+                html = Template.render_twig_template("errors/500.twig",
+                                                     {"server": {"url": url}, "error_message": error_string})
+
+            writer.write(headers + html.encode())
             await writer.drain()
-
-        writer.close()
+            writer.close()
 
     def __init__(self, host_name, port):
         self.content_raw = None
@@ -324,6 +398,7 @@ class Webserver:
         self.port = port
         self.router_handler = None
         self.running = False
+        self.server = None
 
     async def serve_forever(self):
         await self.run_server()
