@@ -10,11 +10,11 @@ import os
 import shutil
 import importlib
 import sys
+import traceback
 import sass
 from pathlib import Path
 from watchdog.observers import Observer
 from watchdog.events import FileSystemEventHandler, FileSystemEvent
-
 from tina4_python.Router import get
 from tina4_python import Messages, Constant
 from tina4_python.Swagger import Swagger
@@ -25,6 +25,7 @@ from tina4_python.Localization import localize
 from tina4_python.Auth import Auth
 from tina4_python.Debug import Debug
 from tina4_python.ShellColors import ShellColors
+from tina4_python.Session import Session
 
 _ = gettext.gettext
 
@@ -51,6 +52,7 @@ if importlib.util.find_spec("jurigged"):
 # Define the variable to be used for global routes
 library_path = os.path.dirname(os.path.realpath(__file__))
 root_path = os.path.realpath(os.getcwd())
+sys.path.append(root_path)
 
 if not os.path.exists(root_path + os.sep + "logs"):
     os.makedirs(root_path + os.sep + "logs")
@@ -65,6 +67,21 @@ tina4_current_request = {}
 tina4_api_key = None
 tina4_auth = Auth(root_path)
 
+
+# Set up the global exception handler
+def global_exception_handler(exception):
+    debug_level = Constant.TINA4_LOG_DEBUG
+    error = str(exception)
+    tb_str = ''.join(traceback.format_exception(None, exception, exception.__traceback__))
+    error_string = "Exception Error: "+ error+"\n"+tb_str+"\nYou are seeing this error because Tina4 is in debug mode"
+    Debug.error(error_string)
+    if (os.getenv("TINA4_DEBUG_LEVEL", [Constant.TINA4_LOG_ALL]) == "[TINA4_LOG_ALL]"
+            or debug_level in os.getenv("TINA4_DEBUG_LEVEL", [Constant.TINA4_LOG_ALL])):
+        pass
+    else:
+        error_string = ""
+    return error_string
+
 token = tina4_auth.get_token({"name": "Tina4"})
 Debug("TEST TOKEN", token, Constant.TINA4_LOG_DEBUG)
 Debug("VALID TOKEN", tina4_auth.valid(token + "a"), Constant.TINA4_LOG_DEBUG)
@@ -115,12 +132,7 @@ if not os.path.exists(root_path + os.sep + "src" + os.sep + "public"):
 # ignore F403
 if os.path.exists(root_path + os.sep + "src"):
     try:
-        src_path = root_path + os.sep + "src"
-        for file in os.listdir(src_path):
-            if file.endswith(".py"):
-                file_name = file.removesuffix(".py")
-                exec("from src import "+ file_name)
-
+        exec("from src import *")
     except ImportError as e:
         Debug("Cannot import src folder", str(e), Constant.TINA4_LOG_ERROR)
 else:
@@ -128,11 +140,7 @@ else:
 
 if os.path.exists(root_path + os.sep + "src" + os.sep + "routes"):
     try:
-        src_path = root_path + os.sep + "src"+ os.sep + "routes"
-        for file in os.listdir(src_path):
-            if file.endswith(".py"):
-                file_name = file.removesuffix(".py")
-                exec("from src.routes import "+ file_name)
+        exec("from src.routes import *")
     except ImportError as e:
         Debug("Cannot import src.routes folder", str(e), Constant.TINA4_LOG_ERROR)
 else:
@@ -140,11 +148,7 @@ else:
 
 if os.path.exists(root_path + os.sep + "src" + os.sep + "app"):
     try:
-        src_path = root_path + os.sep + "src"+ os.sep + "app"
-        for file in os.listdir(src_path):
-            if file.endswith(".py"):
-                file_name = file.removesuffix(".py")
-                exec("from src.app import "+ file_name)
+        exec("from src.app import *")
     except ImportError as e:
         Debug("Cannot import src.app folder", str(e), Constant.TINA4_LOG_ERROR)
 else:
@@ -189,7 +193,7 @@ def file_get_contents(file_path):
 
 
 # Add swagger routes
-@get(os.getenv("SWAGGER_ROUTE", "/swagger")+"/swagger.json")
+@get(os.getenv("SWAGGER_ROUTE", "/swagger") + "/swagger.json")
 async def get_swagger_json(request, response):
     json = Swagger.get_json(request)
     return response(json)
@@ -203,28 +207,151 @@ async def get_swagger(request, response):
     html = html.replace("{SWAGGER_ROUTE}", os.getenv("SWAGGER_ROUTE", "/swagger"))
     return response(html)
 
+async def app(scope, receive, send):
+    """
+    Runs normal hypercorn, uvicorn, granian
+    :param scope:
+    :param receive:
+    :param send:
+    :return:
+    """
+    body = b""
+    while True and scope['type'] == 'http' or scope['type'] == 'websocket':
+        if scope['type'] != 'websocket':
+            message =  await receive()
+        else:
+            message = {'type': 'websocket'}
 
-def webserver(host_name, port):
-    web_server = Webserver(host_name, int(port))  # HTTPServer((host_name, int(port)), Webserver)
-    web_server.router_handler = Router()
-    # Fix the display to make it clickable
-    if host_name == "0.0.0.0":
-        host_name = "localhost"
-    Debug(Messages.MSG_SERVER_STARTED.format(host_name=host_name, port=port), Constant.TINA4_LOG_INFO)
-    try:
-        asyncio.run(web_server.serve_forever())
-    except KeyboardInterrupt:
-        pass
-    web_server.server_close()
-    Debug(Messages.MSG_SERVER_STOPPED, Constant.TINA4_LOG_INFO)
+        if "body" in message:
+            body += message["body"]
+        if message['type'] == 'lifespan.startup':
+            await send({'type': 'lifespan.startup.complete'})
+            return
+        elif message['type'] == 'lifespan.shutdown':
+            await send({'type': 'lifespan.shutdown.complete'})
+            return
+        elif message["type"] == "http.disconnect" or message["type"] == "websocket.disconnect":
+            return
+        elif not message.get("more_body"):
+            webserver = Webserver(scope["server"][0], scope["server"][1])
+            parsed_headers = {}
+            parsed_headers_lowercase = {}
+            for header in scope["headers"]:
+                parsed_headers[header[0].decode()] = header[1].decode()
+                parsed_headers_lowercase[header[0].decode().lower()] = header[1].decode()
+
+            if "content-length" not in parsed_headers_lowercase and "accept" in parsed_headers_lowercase:
+                parsed_headers_lowercase["content-type"] = parsed_headers_lowercase["accept"].split(",")[0]
+                parsed_headers["Content-Type"] = parsed_headers_lowercase["content-type"]
+
+            if "content-length" not in parsed_headers_lowercase:
+                parsed_headers_lowercase["content-length"] = 0
+                parsed_headers_lowercase["Content-Length"] = parsed_headers_lowercase["content-length"]
+
+            webserver.headers = parsed_headers
+            webserver.lowercase_headers = parsed_headers_lowercase
+
+            webserver.path = scope["path"]+"?"+scope["query_string"].decode()
+
+            if "method" in scope:
+                webserver.method = scope["method"]
+            else:
+                webserver.method = "GET"
+
+            if message["type"] == "http.request":
+                webserver.content_raw = body
+            else:
+                webserver.content_raw = b""
+            webserver.content_length = parsed_headers_lowercase["content-length"]
+            webserver.router_handler = Router()
+            webserver.session = Session
+
+            cookie_list = {}
+            if "cookie" in webserver.lowercase_headers:
+                cookie_list_temp = webserver.lowercase_headers["cookie"].split(";")
+                for cookie_value in cookie_list_temp:
+                    cookie = cookie_value.split("=", 1)
+                    cookie_list[cookie[0].strip()] = cookie[1].strip()
+
+            webserver.cookies = cookie_list
+
+            # initialize the session
+            webserver.session = Session(os.getenv("TINA4_SESSION", "PY_SESS"),
+                                   os.getenv("TINA4_SESSION_FOLDER", root_path + os.sep + "sessions"),
+                                   os.getenv("TINA4_SESSION_HANDLER", "SessionFileHandler")
+                                   )
+
+            if os.getenv("TINA4_SESSION", "PY_SESS") in webserver.cookies:
+                webserver.session.load(webserver.cookies[os.getenv("TINA4_SESSION", "PY_SESS")])
+            else:
+                webserver.cookies[os.getenv("TINA4_SESSION", "PY_SESS")] = webserver.session.start()
+
+            tina4_response, tina4_headers = await webserver.get_response(webserver.method, (scope,receive,send),True)
+
+            if message["type"] != "websocket":
+                response_headers = []
+                for header in tina4_headers:
+                    header = header.split(":")
+                    response_headers.append([header[0].strip().encode(), header[1].strip().encode()])
+
+                await send({
+                    'type': 'http.response.start',
+                    'status': tina4_response.http_code,
+                    'headers': response_headers,
+                })
+
+                if isinstance(tina4_response.content, str):
+                    await send({
+                        'type': 'http.response.body',
+                        'body': tina4_response.content.encode(),
+                    })
+                else:
+                    await send({
+                        'type': 'http.response.body',
+                        'body': tina4_response.content,
+                    })
 
 
 def run_web_server(in_hostname="localhost", in_port=7145):
     Debug(Messages.MSG_STARTING_WEBSERVER.format(port=in_port), Constant.TINA4_LOG_INFO)
     webserver(in_hostname, in_port)
 
+def webserver(host_name, port):
+    """
+    Runs the correct webserver
+    :param host_name:
+    :param port:
+    :return:
+    """
+    if os.getenv("TINA4_BUILT_IN_WEBSERVER", "False").upper() == "TRUE":
+        # runs the built-in webserver (websockets) don't work on windows
+        web_server = Webserver(host_name, int(port))  # HTTPServer((host_name, int(port)), Webserver)
+        web_server.router_handler = Router()
+        # Fix the display to make it clickable
+        if host_name == "0.0.0.0":
+            host_name = "localhost"
+        Debug(Messages.MSG_SERVER_STARTED.format(host_name=host_name, port=port), Constant.TINA4_LOG_INFO)
+        try:
+            asyncio.run(web_server.serve_forever())
+        except KeyboardInterrupt:
+            pass
+        web_server.server_close()
+    else:
+        # Runs a hyper corn server
+        try:
+            from hypercorn.config import Config
+            from hypercorn.asyncio import serve
+            config = Config()
+            config.bind = [host_name+":"+str(port)]
+            asyncio.run(serve(app, config))
+        except Exception:
+            Debug("Not running Hypercorn webserver", str(e), Constant.TINA4_LOG_WARNING)
 
-if os.getenv('TINA4_DEFAULT_WEBSERVER', 'True') == 'True':
+
+    Debug(Messages.MSG_SERVER_STOPPED, Constant.TINA4_LOG_INFO)
+
+
+if os.getenv('TINA4_DEFAULT_WEBSERVER', 'True').upper() == 'TRUE':
     if importlib.util.find_spec("jurigged"):
         Debug("Jurigged enabled", Constant.TINA4_LOG_INFO)
         jurigged.watch("./")
@@ -243,8 +370,8 @@ if os.getenv('TINA4_DEFAULT_WEBSERVER', 'True') == 'True':
         try:
             PORT = int(PORT)
             run_web_server(HOSTNAME, PORT)
-        except Exception:
-            Debug("Not running webserver", Constant.TINA4_LOG_WARNING)
+        except Exception as e:
+            Debug("Not running webserver", str(e), Constant.TINA4_LOG_WARNING)
     else:
         Debug("Webserver is set to manual start, please call " + ShellColors.bright_red +
               "run_web_server(<HOSTNAME>, <PORT>)" + ShellColors.end + " in your code",
